#include <gnome.h>
#include <stdio.h>

#include "thunder-presentation.h"

static void
write_line_html (ThunderLine *line)
{
	ThunderType type = thunder_line_get_type (line);
	char *text = thunder_line_get_text (line);
	const char *hlevel = "H1";
	const char *color = NULL;

	if (type == THUNDER_LINE_HEADING) {
		hlevel = "H1";
		color = "black";
	} else if (type == THUNDER_LINE_BULLET_0) {
		hlevel = "H2";
		color = "darkred";
	} else if (type == THUNDER_LINE_BULLET_1) {
		hlevel = "H3";
		color = "darkblue";
	} else if (type == THUNDER_LINE_BULLET_2) {
		hlevel = "H4";
		color = "darkgreen";
	} else if (type == THUNDER_LINE_BULLET_3) {
		hlevel = "H5";
		color = "purple";
	}

	if (type == THUNDER_LINE_IMAGE) {
		printf ("<P><IMG SRC=\"%s\">\n\n", text);
	} else {
		printf ("<P><FONT COLOR=\"%s\"><%s>%s</%s></FONT>\n\n", color, hlevel, text, hlevel);
	}

	g_free (text);
}

static void
write_page_html (ThunderPage *page)
{
	GList *lines, *li;

	lines = thunder_page_get_lines (page);

	for (li = lines; li != NULL; li = li->next) {
		ThunderLine *line = li->data;

		write_line_html (line);
	}

	thunder_line_list_free (lines);

	printf ("<HR NOSHADE>\n\n");
}

static void
write_html (ThunderPresentation *presentation)
{
	GList *pages, *li;

	printf ("<HTML>\n<BODY BGCOLOR=\"#ffffff\">\n<HR NOSHADE>\n");

	pages = thunder_presentation_get_pages (presentation);

	for (li = pages; li != NULL; li = li->next) {
		ThunderPage *page = li->data;

		write_page_html (page);
	}

	thunder_page_list_free (pages);

	printf ("</BODY>\n</HTML>\n");
}

int
main (int argc, char *argv[])
{
	FILE *fp;
	char buf[256];
	ThunderPresentation *presentation;

	gtk_type_init ();
	gtk_signal_init ();

	if (argc != 2) {
		g_warning ("Please provide one argument which is the .ctfdu file");
		return 0;
	}

	presentation = thunder_presentation_new ();

	fp = fopen (argv[1], "r");
	if (fp == NULL) {
		g_warning ("Can't open: %s", argv[1]);
		return 1;
	}

	while (fgets (buf, sizeof (buf), fp) != NULL) {
		thunder_presentation_add_xml (presentation, buf);
	}

	thunder_presentation_parse_xml (presentation);

	write_html (presentation);

	gtk_object_unref (GTK_OBJECT (presentation));

	return 0;
}

