
#include <bonobo.h>

#include "translator.h"

#include "gnome-example-translator.h"

static BonoboObjectClass *parent_class;

static POA_GNOME_Example_Translator__vepv translator_vepv;

static CORBA_char *gnome_example_translator_rot13_string (const CORBA_char *string);


/*
 * CORBA GNOME::Example::Translator::translate implementation
 */
static CORBA_char *
impl_GNOME_Example_Translator_translate (PortableServer_Servant  servant,
					 const CORBA_char       *plaintext,
					 CORBA_Environment      *ev)
{
	GnomeExampleTranslator *translator;

	translator = GNOME_EXAMPLE_TRANSLATOR (bonobo_object_from_servant (servant));
	
	return gnome_example_translator_rot13_string (plaintext);
}


/*
 * CORBA GNOME::Example::Translator::reverseTranslation implementation
 */
static CORBA_char *
impl_GNOME_Example_Translator_reverseTranslation (PortableServer_Servant  servant,
						  const CORBA_char       *cyphertext,
						  CORBA_Environment      *ev)
{
	GnomeExampleTranslator *translator;

	translator = GNOME_EXAMPLE_TRANSLATOR (bonobo_object_from_servant (servant));
	
	return gnome_example_translator_rot13_string (cyphertext);
}


/*
 * Construct the CORBA entry point vector
 */
POA_GNOME_Example_Translator__epv *
gnome_example_translator_get_epv (void)
{
	POA_GNOME_Example_Translator__epv *epv;

	epv = g_new0 (POA_GNOME_Example_Translator__epv, 1);

	epv->translate          = impl_GNOME_Example_Translator_translate;
	epv->reverseTranslation = impl_GNOME_Example_Translator_reverseTranslation;

	return epv;
}

static void
gnome_example_translator_init_corba_class (void)
{
	translator_vepv.Bonobo_Unknown_epv = bonobo_object_get_epv ();
	translator_vepv.GNOME_Example_Translator_epv = gnome_example_translator_get_epv ();
}


/*
 * Function that implements rot13 
 */
static CORBA_char *
gnome_example_translator_rot13_string (const CORBA_char *string)
{
        CORBA_char *rot13_string; 
	int i, length;

	rot13_string = CORBA_string_dup (string);

	length = strlen (rot13_string);

	for (i = 0;  i < length; i++) {
	        if (rot13_string[i] >= 'A' &&
		    rot13_string[i] <= 'Z') {
		        rot13_string[i] = 'A' + ((rot13_string[i] - 'A' + 13) % 26);
		}

	        if (rot13_string[i] >= 'a' &&
		    rot13_string[i] <= 'z') {
		        rot13_string[i] = 'a' + ((rot13_string[i] - 'a' + 13) % 26);
		}
	}

	return rot13_string;
}


/* Gtk/Bonobo boilerplate */ 

static void gnome_example_translator_class_init (GnomeExampleTranslatorClass *klass);
static void gnome_example_translator_destroy    (GtkObject                   *object);

GtkType
gnome_example_translator_get_type (void)
{
	static GtkType type = 0;

	if (!type){
		GtkTypeInfo info = {
			"GnomeExampleTranslator",
			sizeof (GnomeExampleTranslator),
			sizeof (GnomeExampleTranslatorClass),
			(GtkClassInitFunc) gnome_example_translator_class_init,
			(GtkObjectInitFunc) NULL,
			NULL, /* reserved 1 */
			NULL, /* reserved 2 */
			(GtkClassInitFunc) NULL
		};

		type = gtk_type_unique (bonobo_object_get_type (), &info);
	}

	return type;
}


static void
gnome_example_translator_class_init (GnomeExampleTranslatorClass *klass)
{
        GtkObjectClass *object_class = (GtkObjectClass *) klass;

	parent_class = gtk_type_class (bonobo_object_get_type ());

	object_class->destroy = gnome_example_translator_destroy;

	gnome_example_translator_init_corba_class ();
}


GnomeExampleTranslator *
gnome_example_translator_construct (GnomeExampleTranslator  *translator, 
				    GNOME_Example_Translator corba_translator)
{
	g_return_val_if_fail (translator != NULL, NULL);
	g_return_val_if_fail (GNOME_IS_EXAMPLE_TRANSLATOR (translator), NULL);
	g_return_val_if_fail (corba_translator != CORBA_OBJECT_NIL, NULL);

	/*
	 * Call parent constructor
	 */
	if (!bonobo_object_construct (BONOBO_OBJECT (translator), 
				      (CORBA_Object) corba_translator)) {
		return NULL;
	}

	return translator;
}


static GNOME_Example_Translator
gnome_example_translator_create_corba_object (BonoboObject *translator)
{
	POA_GNOME_Example_Translator *servant;
	CORBA_Environment ev;

	servant = (POA_GNOME_Example_Translator *) g_new0 (BonoboObjectServant, 1);
	servant->vepv = &translator_vepv;

	CORBA_exception_init (&ev);
	POA_GNOME_Example_Translator__init ((PortableServer_Servant) servant, &ev);
	ORBIT_OBJECT_KEY(servant->_private)->object = NULL;

	if (ev._major != CORBA_NO_EXCEPTION){
		g_free (servant);
		CORBA_exception_free (&ev);
		return CORBA_OBJECT_NIL;
	}

	CORBA_exception_free (&ev);

	return (GNOME_Example_Translator) bonobo_object_activate_servant (translator, 
									  servant);
}

GnomeExampleTranslator *
gnome_example_translator_new (void)
{
	GnomeExampleTranslator  *translator;
	GNOME_Example_Translator corba_translator;

	translator = gtk_type_new (gnome_example_translator_get_type ());

	corba_translator = gnome_example_translator_create_corba_object 
	  (BONOBO_OBJECT (translator));

	if (corba_translator == CORBA_OBJECT_NIL) {
		bonobo_object_unref (BONOBO_OBJECT (translator));
		return NULL;
	}
	
	return gnome_example_translator_construct (translator, corba_translator);
}

static void
gnome_example_translator_destroy (GtkObject *object)
{
	GnomeExampleTranslator *translator;

	translator = GNOME_EXAMPLE_TRANSLATOR (object);

	GTK_OBJECT_CLASS (parent_class)->destroy (object);
}

